;+
; NAME: 
;    maddownloadfile, madurl, fullFilename, outputFile, user_fullname, user_email, user_affiliation, format
;
; PURPOSE: 
;    downloads a Madrigal file in simple ascii format to local computer
;
; INPUTS: 
;      madurl - scalar string giving a fully qualified url to the Madrigal site
;      fullFilename - full path to experiment file as returned by madGetExperimentFiles.
;      outputFile - path to save file locally
;      user_fullname - full name of user making request
;      user_email - email address of user making request
;      user_affiliation - affiliation of user making request
;      format - 'ascii'or 'hdf5'. Default is ascii.
;
; EXAMPLE: 
;     maddownloadfile, 'http://madrigal.haystack.mit.edu/madrigal', '/opt/madrigal/experiments/1998/mlh/20jan98/mlh980120g.002', '/tmp/junk.hdf5', &
;                                   'Bill Rideout', 'brideout@haystack.mit.edu', 'MIT', 'hdf5'
;
; $Id: maddownloadfile.pro 4782 2015-02-11 16:53:19Z brideout $
;
PRO maddownloadfile, madurl,  fullFilename, outputFile, user_fullname, user_email, user_affiliation, format

    ; determine if needed parameters set
    if (n_params() lt 6) then begin
      message, 'Too few parameters specified - see usage in maddownloadfile.pro'
    endif
    
    if (n_params() lt 7) then begin
        format = 'ascii'
    endif
    
    if ((format NE 'ascii') AND (format NE 'hdf5')) then begin
      message, 'format must be ascii or hdf5 is specified'
    endif
    
    if (format EQ 'ascii') then begin

	    ; the first step is to call madGetExperimentFileParameters to find out what parameters to request
	    parms = madGetExperimentFileParameters(madurl, fullFilename)
	    parmStr = 'year,month,day,hour,min,sec'
	    headerStr = 'year month day hour min sec '
	    numParms = 6
	    for i=0, n_elements(parms)-1 do begin
	        if (parms[i].isMeasured eq 0) then continue ; skip derived parameters
	        if (parms[i].isAddIncrement eq 1) then continue ; skip additional increment
	        parmStr = parmStr + ',' + parms[i].mnemonic
	        headerStr = headerStr + ' ' + parms[i].mnemonic
	        numParms = numParms + 1
	    endfor
	
	    ; now call madprint without filters
	    simpleData = madprint(madurl,  fullFilename, parmStr, '', user_fullname, user_email, user_affiliation)
	     
	    ; build a format string
	    formatStr = '(%"%f '
	    for i=1, numParms-1 do begin
	        formatStr = formatStr + '%f '
	    endfor
	    formatStr = formatStr + '")'
	    
	    ; write result to file
	    openw, 11, outputFile
	    printf, 11, headerStr
	    
	    ; loop through each data row and write it to file
	    result = size(simpleData)
	    for k=0, result[2]-1 do begin
	        printf, 11, FORMAT=formatStr,  simpleData[*,k]
	    endfor
	    
	    close, 11
	    
	endif else begin
	; hdf5 case
	
	cgiUrl = madgetcgiurl(madurl)
	; build command
	cmd = string(format='(%"wget -q -O %s ")', outputFile)
	scriptName = 'getMadfile.cgi'
	expUrl = '"' + cgiUrl + scriptName + '?'
	; append times
	expUrl += string(format='(%"fileName=%s&")', fullFilename)
	expUrl += 'fileType=-2&'
	user_fullname2 = String(Replicate(32B, strlen(user_fullname)))
        strput, user_fullname2, user_fullname
        strreplaceall, user_fullname2, ' ', '+'
        expUrl += string(format='(%"user_fullname=%s&")', user_fullname2)
        user_email2 = String(Replicate(32B, strlen(user_email)))
        strput, user_email2, user_email
        strreplaceall, user_email2, ' ', '+'
        expUrl += string(format='(%"user_email=%s&")', user_email2)
        user_affiliation2 = String(Replicate(32B, strlen(user_affiliation)))
        strput, user_affiliation2, user_affiliation
        strreplaceall, user_affiliation2, ' ', '+'
        expUrl += string(format='(%"user_affiliation=%s&")', user_affiliation2)
	cmd += expUrl
        cmd += '"'
	
	; run command
	SPAWN, cmd, result, errresult
	errresult = STRJOIN(errresult, /SINGLE)
	if (STRLEN(errresult) GT 4) then begin
	    message, 'wget not installed on your system - you need to install it first to run maddownloadfile or madglobaldownload'
	endif
	
	
	endelse
END
